/**
 * @file src/SoilSurfaceIncomingWater.cpp
 * @author R.Trepos-(The RECORD team -INRA )
 */

/*
 * Copyright (C) 2013 INRA
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include <vle/extension/difference-equation/Multiple.hpp>

namespace tp5_2_correction {

    class SoilSurfaceIncomingWater: public vle::extension::DifferenceEquation::Multiple
    {
	public:
	    SoilSurfaceIncomingWater(const vle::devs::DynamicsInit& atom,
		 const vle::devs::InitEventList& events) :
	    vle::extension::DifferenceEquation::Multiple(atom, events)
	    {
		    // Variables d'etat gerees par ce composant
		    Irrigation = createVar("Irrigation");
		    AvailableWater = createVar("AvailableWater"); 
		    RunOff = createVar("RunOff");
		    Infiltration = createVar("Infiltration");
		
		    // Variables  gerees par un autre  composant
		    Rain = createSync("Rain");
		    IncomingRunOff = createSync("IncomingRunOff");
		
		    // Lecture des parametres
		    fracRunOff = (events.exist("fracRunOff")) 
		             ?  events.getDouble("fracRunOff") 
		             :  0;
	    }


	    virtual ~SoilSurfaceIncomingWater() {};


   	    virtual void compute(const vle::devs::Time& /*julianDay*/)
	    {
		    //variables qui peuvent être perturbées
		    Irrigation = 0;
		    //autres variables
		    AvailableWater = Rain() + Irrigation() + IncomingRunOff();
		    RunOff = AvailableWater() * fracRunOff;
		    Infiltration = AvailableWater() * (1 - fracRunOff);
        }


        virtual void initValue(const vle::devs::Time& /* julianDay */)
        {
	    	Infiltration = 0.0;
	    	Irrigation = 0.0;
	    	RunOff = 0.0;
        }


        private:
	    //Variables d'etat
	    Var Irrigation;
	    Var AvailableWater;
	    Var RunOff;
	    Var Infiltration;
	     
	    //Entrées
	    Sync Rain;
	    Sync IncomingRunOff;
	    
	    //Parametres du modele
	    double fracRunOff; 
    };
}//namespaces tp5_2_correction
DECLARE_DYNAMICS(tp5_2_correction::SoilSurfaceIncomingWater); // balise specifique VLE


